package FCC::Actions::Index;
$VERSION = 1.00;
use strict;
use warnings;
use utf8;
use CGI;
use File::Basename;
use Config::Tiny;
use Template;
use URI;

sub new {
    my ( $caller, %args ) = @_;
    my $class = ref($caller) || $caller;
    my $self  = {
        cgi_dir => $args{cgi_dir},
        cgi     => CGI->new,
        conf    => {}
    };
    bless $self, $class;
    return $self;
}

sub run {
    my ($self) = @_;

    # このファイルのパッケージ名
    my @pname_parts = split( /\:\:/, __PACKAGE__ );
    my $pname       = pop @pname_parts;

    # CGI.pm のインスタンス
    my $cgi = CGI->new;
    $self->{cgi} = $cgi;

    # 設定を読み取る
    my $conf_data = $self->load_conf();
    $self->{conf}             = $conf_data->{default};
    $self->{conf}->{CGI_DIR}  = $self->{cgi_dir};
    $self->{conf}->{BASE_DIR} = $self->{conf}->{CGI_DIR} . '/exec';

    # 呼び出された CGI の URL 情報を設定に追加
    my $cgi_url = $cgi->url();
    $self->{conf}->{CGI_URL}  = $cgi_url;    # https://xxx/app/index.cgi
    $self->{conf}->{CGI_PATH} = URI->new($cgi_url)->path;    # /app/index.cgi
    $self->{conf}->{CGI_APP_PATH} =
      File::Basename::dirname( $ENV{SCRIPT_NAME} );          # /app
    $self->{conf}->{CGI_SSL}    = $cgi->https() ? 1 : 0;
    $self->{conf}->{CGI_DOMAIN} = URI->new($cgi_url)->host;

    # static ディレクトリの URL を判定
    my $url = $self->{conf}->{CGI_APP_PATH} . '/static';
    $self->{conf}->{static_url} = $url;

    #テンプレートディレクトリ
    my $tmpl_dir = $self->{conf}->{BASE_DIR} . '/template/' . $pname;
    $self->{conf}->{TEMPLATE_DIR} = $tmpl_dir;

    # 処理モードを取得
    my $mode = $cgi->param('m') || 'index';
    $mode = ucfirst lc $mode;
    $self->{conf}->{MODE} = $mode;

    # テンプレートオブジェクトを生成
    my $tt = Template->new(
        {
            INCLUDE_PATH => $tmpl_dir,
            INTERPOLATE  => 1,
            ENCODING     => 'utf-8',
            DEBUG        => 1
        }
    );

    # 該当ページのモジュールを読み込んで実行
    my $apm;
    my $actions_dpath =
      $self->{conf}->{BASE_DIR} . '/fcclib/FCC/Actions/' . $pname;

    if ( -e "${actions_dpath}/${mode}.pm" ) {
        $apm = "FCC::Actions::${pname}::${mode}";
        eval qq(require $apm; import $apm);
        if ($@) { die $@; }
    }
    else {
        $self->print_error( 404, '指定のページは見つかりませんでした。' );
    }

    my $action = eval "${apm}->new";
    if ($@) {
        die $@;
    }

    $action->set( 'conf',     $self->{conf} );
    $action->set( 'cgi',      $cgi );
    $action->set( 'template', $tt );
    $action->dispatch();
}

sub load_conf {
    my ($self) = @_;

    # /exec/etc/conf.ini.cgi の設定情報を読み取る
    my $base_dir = $self->{cgi_dir} . '/exec';
    my $fpath    = "${base_dir}/etc/conf.ini.cgi";
    my $conf     = Config::Tiny->read( $fpath, 'utf8' )
      or die "failed to read deafult configurations file `${fpath}`. : $!";

    return $conf;
}

sub print_error {
    my ( $self, $code, $msg ) = @_;
    print $self->{cgi}->header(
        -status  => $code,
        -type    => 'text/plain',
        -charset => 'utf-8',
    );
    print $msg;
    exit;
}

1;
